#include<algorithm>
#include<cstdio>
#include<cassert>
#include<iostream>
#include<vector>
#include <map>
using namespace std;

int T, R, C;
char F;
bool swapped;

typedef pair<int,int> pii;

map<pii, vector<pii>> small{
  {{3,3}, {{0,1},{1,0},{2,1},{1,2}}},
  {{3,5}, {{0,1},{1,0},{2,1},{1,2},{0,3},{1,4},{2,3}}},
  {{5,5}, {{0,1},{1,0},{2,1},{3,0},{4,1},{3,2},{4,3},{3,4},{2,3},{1,2},{0,3},{1,4}}},
  {{5,7}, {{0,1},{1,0},{2,1},{3,0},{4,1},{3,2},{2,3},{1,2},{0,3},{1,4},{0,5},{1,6},{2,5},{3,6},{4,5},{3,4},{4,3}}}
};

/*
vector<pair<int, int> > small_instances
3 3
     0   
  1     3
     2   

3 5
     0     4   
  1     3     5
     2     6   

5 5
     0    10   
  1     9    11
     2     8   
  3     5     7
     4     6   

5 7
     0     8    10   
  1     7     9    11
     2     6    12   
  3     5    15    13
     4    16    14   
*/

void output(int r, int c, bool swp) {
  if (swp) swap(r, c);
  cout << (r+1) << " " << (c+1) << endl;
}

void print(bool swp, vector<pair<int, int> >& v) {
  for(int i=0; i<(int)v.size(); ++i) output(v[i].first, v[i].second, swp);
  cout << endl;
}

void print_debug(vector<pair<int,int> > v) {
  int lor, loc, hir, hic;
  lor = hir = v[0].first;
  loc = hic = v[0].second;
  for(int i=0; i<(int)v.size(); ++i) {
    lor = min(lor, v[i].first);
    loc = min(loc, v[i].second);
    hir = max(hir, v[i].first);
    hic = max(hic, v[i].second);
  }
  int ofr = -lor;
  int ofc = -loc;
  vector<vector<int> > M(hir-lor+1, vector<int>(hic-loc+1, -1));
  for(int i=0; i<(int)v.size(); ++i)
    M[ ofr + v[i].first ][ ofc + v[i].second ] = i;
  for(int i=0; i<(int)M.size(); ++i) {
    for(int j=0; j<(int)M[i].size(); ++j)
      if (M[i][j] >= 0) printf("%4d ", M[i][j]);
      else printf("     ");
    printf("\n");
  }
  printf("\n");
}

void add(int r, int c, vector<pair<int, int> >& ret) {
  ret.push_back(make_pair(r,c));
}

void zigzag(int ofr, int ofc, int dir, int steps, vector<pair<int, int> >& ret) {
  int dr[4]  = {  0,  1,  0, -1 };
  int dc[4]  = {  1,  0, -1,  0 };
  int ddr[4] = { -1,  0,  1,  0 };
  int ddc[4] = {  0,  1,  0, -1 };
  for(int s=0; s<steps; ++s) {
    add(ofr, ofc, ret);
    ofr += dr[dir];
    ofc += dc[dir];
    ofr += (2*(-(s%2))+1) * ddr[dir];
    ofc += (2*(-(s%2))+1) * ddc[dir];
  };
}

void ring(int ofr, int ofc, int R, int C, vector<pair<int, int> >& ret) {
  zigzag(ofr+1,   ofc,     0, C, ret);
  zigzag(ofr+2,   ofc+C-2, 1, R-2, ret);
  zigzag(ofr+R-2, ofc+C-3, 2, C-2, ret);
  zigzag(ofr+R-3, ofc+1,   3, R-4, ret);
}

void pattern1(int R, int C, bool swp) {
  vector<pair<int,int> > ret;
  assert(R%2==1 && R==C && R>=7);
  int ofr, ofc, steps;
  ofr = 0, ofc = 0, steps = R;
  while(steps > 3) {
    ring(ofr, ofc, steps, steps, ret);
    ofr += 2;
    ofc += 2;
    steps -= 4;
  }
  if (steps == 3) {
    zigzag(ofr+1, ofc, 0, 3, ret);
    add(ofr+2, ofc+1, ret);
  }
  print(swp, ret);
}

void pattern2(int R, int C, bool swp) {
  vector<pair<int, int> > ret;
  assert(R%2==1 && R+2==C && R>=7);
  swap(R, C); swp = !swp;
  int ofr, ofc, stepr, stepc;
  ofr = 0, ofc = 0, stepr = R; stepc = C;
  while(stepc > 3) {
    ring(ofr, ofc, stepr, stepc, ret);
    ofr += 2;
    ofc += 2;
    stepr -= 4;
    stepc -= 4;
  }
  if (stepc == 1) {
    add(ofr+1, ofc, ret);
  }
  if (stepc == 3) {
    zigzag(ofr+1, ofc, 0, 3, ret);
    zigzag(ofr+2, ofc+1, 1, 3, ret);
    add(ofr+3, ofc, ret);
  }
  print(swp, ret);
} 


int main(void) {
  cin >> T;
  while(T > 0) {
    cin >> R >> C >> F;
    --T;
    assert(F == 'B' || F == 'W');
    if (R > C) { swap(R, C); swapped = true; }
    else swapped = false;
    assert(R * C >= 2);
    if (R == 1 && C <= 2 && F == 'W') { output(0, 0, swapped); cout << endl; continue; }
    if (R == 1 && 2 <= C && C <= 3 && F == 'B') { output(0, 1, swapped); cout << endl; continue; }
    if (R == 2) { for(int i=0; i<C; ++i) output( (i+(F=='B')) % 2, i, swapped); cout << endl; continue; }
    if (R >= 3 && F == 'B' && R % 2 == 1 && (C == R || R+2==C)) {
      if (R == 3 || R == 5) { print(swapped, small[{R, C}]);  continue; }
      if (R >= 7 && R   == C) { pattern1(R, C, swapped); continue; }
      if (R >= 7 && R+2 == C) { pattern2(R, C, swapped); continue; }
    }
    cout << "impossible" << endl << endl;
  }
  return 0;
}
